// 
// The Waveset SPML General License 
// 
// Version 0.1, April 2003
// Copyright (C) 2003 Waveset Technologies, Inc..
// 6034 West Courtyard Drive, Suite 210, Austin, Texas 78730
// All rights reserved.
// 
// TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions, and the disclaimers in Sections 6
//    and 7 below.
// 
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the disclaimers in Sections 6
//    and 7 below in the documentation and/or other materials provided
//    with the distribution.
// 
// 3. The end-user documentation included with the redistribution, if
//    any, must include the following acknowledgment:
// 
//    "This product includes software developed by 
//     Waveset Technologies, Inc. (www.waveset.com)."
// 
//    Alternately, this acknowledgment may appear in the software itself, if
//    and wherever such third-party acknowledgments normally appear.
//  
// 4. The names "Waveset" and "Waveset Technologies, Inc." must not be
//    used to endorse or promote products derived from this software
//    without the prior written permission of Waveset. For written
//    permission, please contact www.waveset.com.
//  
// 5. Products derived from this software may not be called "Waveset",
//    nor may "Waveset" appear in their name, without the prior written
//    permission of Waveset.
//  
// 6. NO WARRANTY 
// 
//    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED
//    OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//    DISCLAIMED.
// 
// 7.  LIMITATION OF LIABILITY
// 
//    IN NO EVENT SHALL THE WAVESET OR ITS LICENSORS BE LIABLE FOR ANY
//    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
//    GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
//    IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
//    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// End of Terms and Conditions
// ====================================================================
//
// Component Name: org.openspml.message.BatchResponse
//
// Author(s): Jeff Larson
//
// Description:
//
// Class representing a "batch" of SpmlResponses.  Typically there
// will be only one, but the standard allows batches.
//
//

package org.openspml.message;

import java.util.*;

import org.openspml.util.*;

/**
 * A class representing a <code>batchResponse</code> message.
 * <p>
 * This contains a list of <code>SpmlResponse</code> objects,
 * and an optional set of attributes that applies to the
 * batch as a whole.
 */
public class BatchResponse extends SpmlResponse {

    //////////////////////////////////////////////////////////////////////
    //
    // Fields
    //
    //////////////////////////////////////////////////////////////////////

    static final String ELEMENT = "batchResponse";

    /**
     * List of SpmlResponse objects.
     */
    List _responses;

    /**
     * Optional list of Attribute objects that apply to the batch as a whole.
     */
    List _attributes;

    //////////////////////////////////////////////////////////////////////
    //
    // Constructors
    //
    //////////////////////////////////////////////////////////////////////

    public BatchResponse() {
    }

    public BatchResponse(XmlElement e) {
	parseXml(e);
    }

    public void setAttributes(List l) {
        _attributes = l;
    }

    public void setAttribute(Attribute a) {
        if (_attributes == null)
            _attributes = new ArrayList();
        _attributes.add(a);
    }
    
    public void setAttribute(String name, Object value) {
        setAttribute(new Attribute(name, value));
    }

    public void setResponses(List l) {
	_responses = l;
    }

    public void addResponse(SpmlResponse r) {
	if (_responses == null)
	    _responses = new ArrayList();
	_responses.add(r);
    }

    //////////////////////////////////////////////////////////////////////
    //
    // Accessors
    //
    //////////////////////////////////////////////////////////////////////

    public List getAttributes() {
	return _attributes;
    }

    public List getResponses() {
	return _responses;
    }

    /**
     * Return a response of a given class.
     */
    public SpmlResponse getResponse(Class c) {

        SpmlResponse found = null;
        if (_responses != null) {
            for (int i = 0 ; i < _responses.size() ; i++) {
                SpmlResponse r = (SpmlResponse)_responses.get(i);
                if (r.getClass() == c) {
                    found = r;
                    break;
                }
            }
        }

        return found;
    }

    //////////////////////////////////////////////////////////////////////
    //
    // XML
    //
    //////////////////////////////////////////////////////////////////////

    String getElementName() {
        return ELEMENT;
    }

    void addSubclassElements(SpmlBuffer b) {

	if (_responses != null) {
	    Iterator it = _responses.iterator();
	    while (it.hasNext()) {
		SpmlResponse r = (SpmlResponse)it.next();
		r.toXml(b);
	    }
	}

	Attribute.toXml(b, "batchAttributes", _attributes);
    }

    void parseXml(XmlElement e) {

        super.parseXml(e);

	for (XmlElement child = e.getChildElement() ; 
	     child != null ; child = child.next()) {

	    String tag = child.getLocalName();

	    if (tag.equals("batchAttributes"))
		_attributes = Attribute.parseList(child);
	    else {
                SpmlResponse res = SpmlResponse.parseResponse(child);
                if (res != null)
                    addResponse(res);
            }
        }
    }

}

